#!/usr/bin/perl
#
# Sensor for gathering hardware data for Zabbix via
# IPMI.
# Copyright (c) 2008 Jakov Sosic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Changes and Modifications
# * Tue Mar 17 18:06:11 CET 2009
# 	- rewritten sender to use only one connection
# 	- fixed alarms
# * Thu May 15 03:40:39 CEST 2008       
#       - script created
#

# zabbix path
my $zabbix_sender="/usr/bin/zabbix_sender";
my $zabbix_confd="/etc/zabbix/zabbix_agentd.conf";
my $send_file="/var/tmp/zabbixSenderFileIPMI";
my $zabbix_send_command="$zabbix_sender -c $zabbix_confd -i $send_file";
my $inputString = '';
my $hostname = `/bin/hostname -f`;
chomp($hostname);
my $ipmitool="/usr/bin/ipmitool";

# signal handling
$SIG{'ALRM'} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

local $SIG{TERM} = sub {
	local $SIG{TERM} = 'IGNORE';
	kill TERM => -$$;
	print "0\n";
	exit(-1);
};

# fetch status from mod_status
alarm 25;
my $cmd = "sudo $ipmitool sdr list";
my $server_status = qx($cmd);

# get information from status output
my($ChassisTemp, $Temp1,$Temp2,$Temp3,$Temp4,$Temp5,$Temp6,$Temp7,$Temp8,$Temp9, $Fan1,$Fan2,$Fan3,$Fan4, $Fan5, $Fan6);

$Temp1 = $1 if ($server_status =~ s/Temp 1\s+\|\s+([\d]+)\s+degrees//);
$Temp2 = $1 if ($server_status =~ s/Temp 2\s+\|\s+([\d]+)\s+degrees//);
$Temp3 = $1 if ($server_status =~ s/Temp 3\s+\|\s+([\d]+)\s+degrees//);
$Temp4 = $1 if ($server_status =~ s/Temp 4\s+\|\s+([\d]+)\s+degrees//);
$Temp5 = $1 if ($server_status =~ s/Temp 5\s+\|\s+([\d]+)\s+degrees//);
$Temp6 = $1 if ($server_status =~ s/Temp 6\s+\|\s+([\d]+)\s+degrees//);
$Temp7 = $1 if ($server_status =~ s/Temp 7\s+\|\s+([\d]+)\s+degrees//);
$Temp8 = $1 if ($server_status =~ s/Temp 8\s+\|\s+([\d]+)\s+degrees//);
$Temp9 = $1 if ($server_status =~ s/Temp 9\s+\|\s+([\d]+)\s+degrees//);

$Fan1 = $1 if ($server_status =~ /Fan 1 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan2 = $1 if ($server_status =~ /Fan 2 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan3 = $1 if ($server_status =~ /Fan 3 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan4 = $1 if ($server_status =~ /Fan 4 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan5 = $1 if ($server_status =~ /Fan 5 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
$Fan6 = $1 if ($server_status =~ /Fan 6 .+\|\s+([\d]+[\.]?[\d]*)\s+/);
#print $Temp1."--\n";


# send information to Zabbix server
$inputString .= $hostname ." ipmi.temp.1 ". $Temp1 ."\n" if($Temp1);
$inputString .= $hostname ." ipmi.temp.2 ". $Temp2 ."\n" if($Temp2);
$inputString .= $hostname ." ipmi.temp.3 ". $Temp3 ."\n" if($Temp3);
$inputString .= $hostname ." ipmi.temp.4 ". $Temp4 ."\n" if($Temp4);
$inputString .= $hostname ." ipmi.temp.5 ". $Temp5 ."\n" if($Temp5);
$inputString .= $hostname ." ipmi.temp.6 ". $Temp6 ."\n" if($Temp6);
$inputString .= $hostname ." ipmi.temp.7 ". $Temp7 ."\n" if($Temp7);
$inputString .= $hostname ." ipmi.temp.8 ". $Temp8 ."\n" if($Temp8);
$inputString .= $hostname ." ipmi.temp.9 ". $Temp9 ."\n" if($Temp9);
$inputString .= $hostname ." ipmi.fan.1 ".  $Fan1  ."\n" if($Fan1);
$inputString .= $hostname ." ipmi.fan.2 ".  $Fan2  ."\n" if($Fan2);
$inputString .= $hostname ." ipmi.fan.3 ".  $Fan3  ."\n" if($Fan3);
$inputString .= $hostname ." ipmi.fan.4 ".  $Fan4  ."\n" if($Fan4);
$inputString .= $hostname ." ipmi.fan.5 ".  $Fan5  ."\n" if($Fan5);
$inputString .= $hostname ." ipmi.fan.6 ".  $Fan6  ."\n" if($Fan6);

# write everything to file
open FH, ">", $send_file or die("Can not open file $send_file!");
print FH $inputString;
close(FH);

# finally, send the data and quit
if ( qx($zabbix_send_command) =~ /Failed(:)? 0/i ) { print"0\n"; exit(-1);}
print "1\n";
exit(0);
# EOF
